/**
--
-- RESOURCE_PROVIDERS
--
-- Stores a cross reference between resource provider IDs, which are used as
-- the primary keys for the resource provider entity beans. Also stores 
-- reconciliation data for each resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RESOURCE_DN - DN for the resource the resource provider is responsible for.
-- RECON_STATUS - Whether a reconciliation is currently running (0 = not reconciling,
--                1 = currently reconciling).
-- LAST_RECON_TIME - The amount of time the last reconcilation took.
-- MAX_RECON_DURATION - maximum duration allowed before timeout of the current 
--                      reconcilation (minutes).
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
-- REQUEST_ID - Tracks the process locking the service
-- CURRENT_REQUEST_COUNT - current number of executing requests
-- MAX_REQUEST_COUNT - maximum number of concurrent executing requests (or -1 = unlimited)
-- LAST_RESPONSE_TIME - timestamp of last response (to detect failed resources)
-- RESOURCE_STATUS - resource status (0 = ok, 1 = failed, 2 = failed service being tested)
*/
CREATE TABLE RESOURCE_PROVIDERS 
  (
  PROVIDER_ID 			NVARCHAR(20) NOT NULL,
  RESOURCE_DN 			NVARCHAR(200),
  RECON_STATUS 			SMALLINT,
  LAST_RECON_TIME		DATETIME,
  MAX_RECON_DURATION 		BIGINT,
  LOCK_SERVICE                  SMALLINT,
  REQUEST_ID                    VARCHAR(20),
  CURRENT_REQUEST_COUNT		BIGINT,
  MAX_REQUEST_COUNT		BIGINT,
  LAST_RESPONSE_TIME		DATETIME,
  RESOURCE_STATUS		SMALLINT,
CONSTRAINT RESOURCE_PROVIDERS_PK PRIMARY KEY(PROVIDER_ID)
  )

/**
--
-- REMOTE_SERVICES_REQUESTS
--
-- Table of requests that were made in an asynchronous fashion, or were
-- made while a reconciliation was ongoing.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
--
-- REQUEST_ID - the ID of the request that was made.
--
-- TYPE - The type of request (GENERIC_REQUEST = 0, ASYNCHRONOUS_REQUEST = 1,
--                             INTRA_RECON_REQUEST = 2, SERVICE_DEFERRED_REQUEST = 3)
--
-- OPERATION - (NOOP = 0, ADD = 1, MODIFY = 2, DELETE = 3, SUSPEND = 4, RESTORE = 5, CHANGE_PASSWORD = 6) 
--
-- REQUEST_TIME - The time the request was made.
--
-- EXPIRATION_TIME - The time the request expires. If null the request never expires. (Currently not used.)
--
-- TARGET - The owner of the account for add or the dn for other operations
--
-- SERVICE_DN - The distinguished name of the service instance in the directory
--
-- DATA - The data for the request (attribute values for add and modify).  Serialized Java Collection.
--
-- CONNECTION_POINT - The callback to complete the workflow process.  Serialized Java object.
--
*/
CREATE TABLE REMOTE_SERVICES_REQUESTS 
  (
  PROVIDER_ID 		NVARCHAR(20),
  REQUEST_ID 		NVARCHAR(20) NOT NULL,
  TYPE 			SMALLINT,
  OPERATION		SMALLINT,
  REQUEST_TIME 		DATETIME,
  EXPIRATION_TIME 	DATETIME,
  TARGET 		NVARCHAR(2000),
  SERVICE_DN            NVARCHAR(2000) NOT NULL,
  DATA			NTEXT,
  CONNECTION_POINT      IMAGE,
CONSTRAINT REMOTE_SVR_REQ_FK FOREIGN KEY (PROVIDER_ID)
  REFERENCES RESOURCE_PROVIDERS (PROVIDER_ID),
CONSTRAINT REMOTE_SVR_REQ_PK PRIMARY KEY(REQUEST_ID) 
   )
/**
--
-- REMOTE_RESOURCES_RECONS
--
-- Table of reconciliation units associates with a given resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- DAY_OF_MONTH - Number indicating which day of the month the recon should be
--                run on.
-- MONTH_NUM - Number indicating which month the recon should be run on.
-- DAY_OF_WEEK - Number indicating which day of the week the recon should be run on.
-- DAY_OF_WEEK - Number indicating which hour of the day the recon should be run on.
-- MINUTE_NUM - Number indicating which minute of the hour the recon should be run on.
-- MAX_DURATION - The maximum allowed duration allowed before the recon will time out (minutes)
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
--
*/
CREATE TABLE REMOTE_RESOURCES_RECONS 
  (
  PROVIDER_ID 		NVARCHAR(20) NOT NULL,
  RECON_ID 		BIGINT  NOT NULL,
  DAY_OF_MONTH          SMALLINT,
  MONTH_NUM             SMALLINT,
  DAY_OF_WEEK           SMALLINT,
  HOUR_NUM              SMALLINT,
  MINUTE_NUM            SMALLINT,
  MAX_DURATION		BIGINT,
  LOCK_SERVICE          SMALLINT DEFAULT 1,
  CONSTRAINT REMOTE_RES_RECONS_FK FOREIGN KEY(PROVIDER_ID) 
    REFERENCES RESOURCE_PROVIDERS(PROVIDER_ID),
  CONSTRAINT REMOTE_RES_RECONS_PK PRIMARY KEY(PROVIDER_ID, RECON_ID) 
   )
/**
--
-- REMOTE_RESOURCES_RECON_QUERIES
--
-- Table of reconciliation queries associates with a given reconciliation unit.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- QUERY_ID - Unique ID for each reconcialation query.
-- RECON_FILTER - Filter associated with this reconciliation query.
-- RECON_BASE - Search base associated with this reconciliation query.
-- MAX_DURATION - not used
-- MAX_ENTRIES - not used
-- ATTRIBUTES - attributes to return in a reconciliation
--
*/
CREATE TABLE REMOTE_RESOURCES_RECON_QUERIES 
  (
  PROVIDER_ID 		NVARCHAR(20) NOT NULL,
  RECON_ID 		BIGINT  NOT NULL,
  QUERY_ID 		BIGINT  NOT NULL,
  RECON_FILTER 		NVARCHAR(1000),
  RECON_BASE 		NVARCHAR(1000),
  MAX_DURATION 		BIGINT,
  MAX_ENTRIES 		BIGINT,
  ATTRIBUTES            NTEXT,	
CONSTRAINT REMOTE_RECON_QUERIES_FK FOREIGN KEY(PROVIDER_ID, 
                                               RECON_ID) 
    REFERENCES REMOTE_RESOURCES_RECONS(PROVIDER_ID, 
                                              RECON_ID),
CONSTRAINT REMOTE_RECON_QUERIES_PK PRIMARY KEY(PROVIDER_ID, 
                                               RECON_ID, 
                                               QUERY_ID) 
   )

CREATE TABLE SCHEDULED_MESSAGE (
  SCHEDULED_TIME                        BIGINT,
  SCHEDULED_MESSAGE_ID                  BIGINT NOT NULL,
  MESSAGE                               NTEXT,
  SERVER                                NVARCHAR(20),
  CHECKPOINT_TIME                       BIGINT,
  REFERENCE_ID                          BIGINT,
CONSTRAINT SCHEDULED_MESSAGE_PK PRIMARY KEY(
                                       SCHEDULED_MESSAGE_ID) 
   )

CREATE INDEX SCHEDULED_MESSAGE_TIME_X ON SCHEDULED_MESSAGE (SCHEDULED_TIME, SERVER)

CREATE INDEX SCHEDULED_MESSAGE_CHECKPOINT_X ON SCHEDULED_MESSAGE (CHECKPOINT_TIME)

CREATE INDEX SCHEDULED_MESSAGE_REFERENCE_X ON SCHEDULED_MESSAGE (REFERENCE_ID)

/**
-- 
-- NEXTVALUE
-- Table to keep the current maximum value of primary key integer. It only contains
-- one record, for each identfier
--
-- ID -- The next primary key id to be used
--
*/
CREATE TABLE NEXTVALUE 
    (
    ID                  NVARCHAR(50) NOT NULL,
    NEXTID 		INT NOT NULL,
CONSTRAINT NEXTVALUE_PK PRIMARY KEY(ID)
    )


/**
--
-- PROCESS
-- 
-- This table stores all pending, running and historical requests submitted 
-- to the enrole workflow system. Each request is represented as a process.
--
-- ID - The process id
-- PARENT_ID - The parent process id if there is any
-- NAME - The process name
-- TYPE - The process type
-- DEFINITION_ID - The process definition ID
-- REQUESTER_TYPE - The requester type
-- REQUESTER  - The requester identity
-- REQUESTER_NAME - The requester name
-- DESCRIPTION - The process description
-- PRIORITY - The process priority
-- SCHEDULED - The scheduled starting time of the process
-- STARTED - The starting time of the process
-- COMPLETED - The completed time of the process
-- LASTMODIFIED - The last modified time of the process
-- SUBMITTED - The time the process is submitted
-- STATE - The process state
-- COMMENTS - The comments for the process
-- RESULT_SUMMARY - The result summary code of the process
-- RESULT_DETAIL - The result detail information of the process
-- TENANT - The DN of the tenant the requester beNTEXTs to
--
*/
CREATE TABLE PROCESS 
      (
                ID                      BIGINT NOT NULL,
                PARENT_ID               BIGINT,
                PARENT_ACTIVITY_ID      BIGINT,
                NAME                    NVARCHAR(100),
                TYPE                    NVARCHAR(2) NOT NULL,
                DEFINITION_ID           NVARCHAR(200) NOT NULL,
                REQUESTER_TYPE          NVARCHAR(2),
                REQUESTER               NVARCHAR(200),
	 	REQUESTER_NAME	    	NVARCHAR(100),
                DESCRIPTION             NVARCHAR(300),
                PRIORITY                SMALLINT,
                SCHEDULED               NVARCHAR(50),
                STARTED                 NVARCHAR(50),
                COMPLETED               NVARCHAR(50),
                LASTMODIFIED            NVARCHAR(50),
                SUBMITTED	    	NVARCHAR(50),
                STATE                   NVARCHAR(1),
                NOTIFY               	INT,
                REQUESTEE               NVARCHAR(200),
	  	REQUESTEE_NAME	    	NVARCHAR(100),
                SUBJECT                 NVARCHAR(500),
                COMMENTS                NVARCHAR(200),
                RESULT_SUMMARY          NVARCHAR(2),
                RESULT_DETAIL           NTEXT,
                TENANT			NVARCHAR(200),
CONSTRAINT PROCESS_PK PRIMARY KEY(ID)
    )
    
CREATE INDEX PROC_STATE_X ON PROCESS(STATE)
CREATE INDEX PROCESS_VPC1_X ON PROCESS (PARENT_ID,SUBMITTED,ID,STATE)


/**  
--
-- ACTIVITY
--
-- This table contains records to keep track of each process's execution status. 
--
-- ID - The activity id
-- PROCESS_ID - The process id that the activity beNTEXTs to
-- DEFINITION_ID - The activity definition ID
-- ACTIVITY_INDEX - The index of the activity if it is inside a loop
-- SUBPROCESS_ID - The ID of the subprocess associated with the activity
-- NAME - The activity name
-- DESCRIPTION - The activity description
-- TYPE - The activity type
-- SUBTYPE - The activity subtype
-- PRIORITY - The activity priority (Currently not used)
-- STARTED - The time the activity started
-- COMPLETED - The time the activity completed
-- LASTMODIFIED - The time the activity was last modified
-- STATE - The activity state
-- RESULT_SUMMARY - The result summary code for the activity
-- RESULT_DETAIL - The result detail for the activity
--
*/
CREATE TABLE ACTIVITY (
                ID                      BIGINT NOT NULL,
                PROCESS_ID              BIGINT NOT NULL,
                DEFINITION_ID           NVARCHAR(100) NOT NULL,
                ACTIVITY_INDEX          INT,
                LOOP_COUNT              INT,
                LOOP_RUNCOUNT           INT,
                RETRY_COUNT             INT,
                LOCK_COUNT		INT,
	 	SUBPROCESS_ID   	BIGINT,
                NAME                    NVARCHAR(100),
                DESCRIPTION             NVARCHAR(300),
                TYPE                    NVARCHAR(1),
                SUBTYPE                 NVARCHAR(2),
                PRIORITY                SMALLINT,
                STARTED                 NVARCHAR(50),
                COMPLETED               NVARCHAR(50),
                LASTMODIFIED            NVARCHAR(50),
                STATE                   NVARCHAR(1),
                RESULT_SUMMARY          NVARCHAR(2),
                RESULT_DETAIL           NTEXT,
CONSTRAINT ACTIVITY_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT ACTIVITY_PK PRIMARY KEY(ID)
    )

CREATE UNIQUE INDEX ACTIVITY_NATURAL_KEY ON ACTIVITY (PROCESS_ID, DEFINITION_ID, ACTIVITY_INDEX)

/**  
--
-- PROCESSDATA
--
-- This table keeps the runtime process data of a process
-- After process completion, this table will be removed
-- 
-- ID - The process data ID
-- PROCESS_ID - The process ID associated with the data
-- ACTIVITY_ID - The activity ID associated with the data if there is any
-- DEF_ID - The definition ID for the corresponding relevant data in process definition
-- NAME - The data name
-- DESCRIPTIOn - The data description
-- TYPE - Data type
-- REQUESTER_NAME	- The requester name for user created data
-- REQUESTER_TYPE - The requester type
-- LENGTH - Data length
-- VALUE - Data value
-- 
*/
CREATE TABLE PROCESSDATA 
   (
                PROCESS_ID              BIGINT NOT NULL,
                DEF_ID                  NVARCHAR(100),
                NAME                    NVARCHAR(100),
                CONTEXT                    NVARCHAR(100),
                DESCRIPTION             NVARCHAR(300),
                TYPE			NVARCHAR(500),
                COLLECTION_TYPE			NVARCHAR(500),
                VALUE                   NTEXT,
    		SMALL_VALUE 		NTEXT,
CONSTRAINT PROCESSDATA_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PROCESSDATA_PK PRIMARY KEY(PROCESS_ID, DEF_ID)
   )
/**   
--
-- PROCESSLOG
--
-- This table keeps records of audit events associated with a process
-- 
-- ID - The processlog ID
-- PROCESS_ID - The ID of the process associated
-- ACTIVITY_ID - The ID of the activity associated
-- CREATED - Time the log record created
-- EVENTTYPE - Log event type
-- OLD_PARTICIPANT_TYPE - Old participant type for assignment change event
-- OLD_PARTICIPANT_ID - Old participant ID for assignment change event
-- NEW_PARTICIPANT_TYPE - New participant type for assignment change event
-- NEW_PARTICIPANT_ID - New participant ID for assignment change event
-- REQUESTOR_TYPE - The requester type for any user related event
-- REQUESTOR - The requester ID for any user related event
-- OLD_STATE - The old state for state change event
-- NEW_STATE - The new state for state change event
-- DATA_ID - The data id for data change event
-- NEW_DATA - The data value for data change event
--
*/
CREATE TABLE PROCESSLOG 
   (
                ID                      BIGINT NOT NULL,
                PROCESS_ID              BIGINT,
                ACTIVITY_ID             BIGINT,
                CREATED                 NVARCHAR(50),
                EVENTTYPE               NVARCHAR(2) NOT NULL,
                OLD_PARTICIPANT_TYPE    NVARCHAR(2),
                OLD_PARTICIPANT_ID      NVARCHAR(200),
                NEW_PARTICIPANT_TYPE    NVARCHAR(2),
                NEW_PARTICIPANT_ID      NVARCHAR(200),
                REQUESTOR_TYPE          NVARCHAR(2),
                REQUESTOR               NVARCHAR(200),
                OLD_STATE               NVARCHAR(1),
                NEW_STATE               NVARCHAR(1),
                DATA_ID                 NVARCHAR(100),
                NEW_DATA                NTEXT,
CONSTRAINT PROCESSLOG_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PROCESSLOG_PK PRIMARY KEY(ID)
    )
/**  
--
-- WORKITEM
-- 
-- This table keeps records of workitems associated with manual workflow activities
-- for running process. The records associated with a process will be removed after 
-- process completion
--
-- ID  -  The workitem ID
-- PROCESS_ID - The process ID associated with the workitem
-- ACTIVITY_ID - The activity ID assciated with the workitem
-- PARTICIPANT_TYPE - The workitem participant type
-- PARTICIPANT  -  The workitem participant identity
-- CREATED - The time the workitem created
--   
*/
CREATE TABLE WORKITEM 
  (
                ID                      BIGINT NOT NULL,
                PROCESS_ID              BIGINT NOT NULL,
                ACTIVITY_ID             BIGINT NOT NULL,
                PARTICIPANT_TYPE        NVARCHAR(2),
                PARTICIPANT             NVARCHAR(2000),
                CREATED                 NVARCHAR(50),
                INPUT_PARAMETERS        NTEXT,
CONSTRAINT WORKITEM2_FK FOREIGN KEY(ACTIVITY_ID) 
    REFERENCES ACTIVITY(ID),
CONSTRAINT WORKITEM_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT WORKITEM_PK PRIMARY KEY(ID)
   )

CREATE TABLE PASSWORD_TRANSACTION 
  (
        TRANSACTION_ID 		BIGINT NOT NULL,
  	ACCOUNT_DN 		NVARCHAR(2000),
  	CREATION_DATE		NVARCHAR(50),
	PROCESS_ID		BIGINT NOT NULL,
	ACTIVITY_ID		BIGINT NOT NULL,
	PASSWORD		NVARCHAR(500),
CONSTRAINT PASSWORD_TRANSACTION_PK PRIMARY KEY(TRANSACTION_ID) 
  )

/**  
--
-- PENDING
--
-- This table keeps the pending provisioning processes
-- 
-- PROCESS_ID - The process ID associated with the data
-- PERSON_DN  - The definition ID for the corresponding relevant data in process definition
-- SERVICE_DN - The activity ID associated with the data if there is any
-- 
*/
CREATE TABLE PENDING 
(
    PROCESS_ID              BIGINT NOT NULL,
    PERSON_DN               NVARCHAR(2000),
    SERVICE_DN              NVARCHAR(2000),
CONSTRAINT PENDING_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PENDING_PK PRIMARY KEY(PROCESS_ID)
)


/**
--
-- LISTDATA
--
-- This table is used to persist large data list for performance optimazation
--
-- DATA_ID - The unique identifier of the data
-- INDEX   - The index of the list element
-- VALUE   - The serialized list element
--
*/
CREATE TABLE LISTDATA 
(
    DATA_ID     BIGINT NOT NULL,
    INDEX_ID    INT NOT NULL,
    VALUE       NTEXT,
CONSTRAINT LISTDATA_PK PRIMARY KEY(DATA_ID, INDEX_ID)
)

CREATE INDEX PENDING_FOR_PERSON ON PENDING (PERSON_DN)

CREATE TABLE REPORT (
	ID INT NOT NULL,
	TITLE VARCHAR(255) NOT NULL, 
	TYPE VARCHAR(255), 
	AUTHOR VARCHAR(255), 
	REPORT_SIZE INT,
	REPORT_DATA IMAGE,
	STYLESHEET_NAME VARCHAR(255), 

	CONSTRAINT REPORT_P PRIMARY KEY (ID) 
)

CREATE TABLE ENTITY_COLUMN (
	ENTITY_NAME VARCHAR(255) NOT NULL, 
	COLUMN_NAME VARCHAR(255) NOT NULL, 
	ATTRIBUTE_NAME VARCHAR(255) NOT NULL, 
	MULTI_VALUED CHAR(1), 
	AVAILABLE_FOR_REPORTING VARCHAR (255), 
	TABLE_NAME VARCHAR (255),
	CONSTRAINT ENTITY_COLUMNP PRIMARY KEY (ENTITY_NAME, ATTRIBUTE_NAME)
)

CREATE TABLE COLUMN_REPORT (
	ENTITY_NAME VARCHAR(255) NOT NULL, 
	COLUMN_NAME VARCHAR(255) NOT NULL, 
	REPORT_ID INT NOT NULL, 
	
	CONSTRAINT COLUMN_REPORT_P PRIMARY KEY (REPORT_ID, ENTITY_NAME, COLUMN_NAME)
)

CREATE TABLE ACI (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    SCOPE VARCHAR(255),
    TARGET VARCHAR(255) NOT NULL,
    PARENT VARCHAR(255),
    PRIMARY KEY(DN, NAME, TARGET)
)

CREATE TABLE ACI_ROLEDNS (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ROLEDN VARCHAR(255) NOT NULL
)

CREATE TABLE ACI_PRINCIPALS (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    PRINCIPALNAME VARCHAR(255) NOT NULL
)

CREATE TABLE ACI_PERMISSION_ATTRIBUTERIGHT (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ACTION VARCHAR(6) NOT NULL,
    OPERATION VARCHAR(3) NOT NULL,
    ATTRIBUTERIGHT VARCHAR(255) NOT NULL
)

CREATE TABLE ACI_PERMISSION_CLASSRIGHT (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ACTION VARCHAR(6) NOT NULL,
    CLASSRIGHT VARCHAR(255) NOT NULL
)

CREATE TABLE ENTITLEMENT (
    DN VARCHAR(255) NOT NULL,
    TYPE VARCHAR(255) NOT NULL,
    SERVICETARGETTYPE VARCHAR(255),
    SERVICETARGETNAME VARCHAR(255),
    PROCESSDN VARCHAR(255)
)

CREATE TABLE ENTITLEMENT_PROVISIONINGPARAMS (
    DN VARCHAR(255) NOT NULL,
    ATTRIBUTEVALUE VARCHAR(4000) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    ENFORCEMENT VARCHAR(255),
    EXPRTYPE VARCHAR(255)
)

CREATE TABLE SYNCHRONIZATION_HISTORY (
    SYNC_ID BIGINT  NOT NULL ,
    REQUESTOR VARCHAR (255) NOT NULL,
    REQ_TYPE VARCHAR (255),
    REQ_NAME VARCHAR (255),
    STATUS VARCHAR (255),
    TENANT VARCHAR (255),
    STATUS_DETAIL VARCHAR (255),
    SCHEDULED_TIME BIGINT,
    SUBMITTED_TIME BIGINT,
    STARTED_TIME BIGINT NOT NULL,
    COMPLETED_TIME BIGINT,
    SERVER_NAME VARCHAR (255),
    PRIMARY KEY (SYNC_ID, STARTED_TIME)
)

CREATE TABLE SYNCHRONIZATION_LOCK (
    HOST VARCHAR (255) NOT NULL,
    PRIMARY KEY (HOST)
)

CREATE TABLE RESOURCES_SYNCHRONIZATIONS (
    SYNC_ID BIGINT  NOT NULL,
    DAY_OF_MONTH INTEGER  NOT NULL,
    MONTH_NUM INTEGER  NOT NULL,
    DAY_OF_WEEK INTEGER  NOT NULL,
    HOUR_NUM INTEGER  NOT NULL,
    MINUTE_NUM INTEGER  NOT NULL,
    MAX_DURATION BIGINT,
    PRIMARY KEY (SYNC_ID)
)

CREATE TABLE CHANGELOG(
CHANGE_NUMBER INTEGER NOT NULL
)

/**
--
-- AUTH_KEY
--
-- This table is used to store the keys for signing and verifying authentication requests.
--
-- Y   - The public key in the DSA algorithm
-- P   - The prime number in the DSA algorithm
-- Q   - The sub-prime number in the DSA algorithm
-- G   - The modulus in the DSA algorithm
-- X   - The private key in the DSA algorithm
--
*/
CREATE TABLE AUTH_KEY (
  Y NVARCHAR(2000) NOT NULL,
  P NVARCHAR(2000) NOT NULL,
  Q NVARCHAR(2000) NOT NULL,
  G NVARCHAR(2000) NOT NULL,
  X NVARCHAR(2000) NOT NULL
) 

CREATE TABLE RECONCILIATION (
  RECONID VARCHAR(255) NOT NULL,
  SERVICEDN VARCHAR(2000) NOT NULL,
  PROCESSEDACCOUNTS INTEGER NOT NULL,
  LOCALACCOUNTS INTEGER NOT NULL,
  TIMUSERACCOUNTS INTEGER NOT NULL,
  POLICYVIOLATIONS INTEGER NOT NULL,
  STARTED DATETIME NOT NULL,
  COMPLETED DATETIME NOT NULL,
  PRIMARY KEY (RECONID)
)

CREATE TABLE RECONCILIATION_INFO (
  RECONID VARCHAR(255) NOT NULL,
  ACCOUNTID VARCHAR(255),
  POLICYCOMPLIANCESTATUS VARCHAR(20),
  USERNAME VARCHAR(255),
  OPERATION VARCHAR(20),
  REMARKS VARCHAR(1000),
  FOREIGN KEY (RECONID) REFERENCES RECONCILIATION (RECONID)
)