/**
--
-- RESOURCE_PROVIDERS
--
-- Stores a cross reference between resource provider IDs, which are used as
-- the primary keys for the resource provider entity beans. Also stores 
-- reconciliation data for each resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RESOURCE_DN - DN for the resource the resource provider is responsible for.
-- RECON_STATUS - Whether a reconciliation is currently running (0 = not reconciling,
--                1 = currently reconciling).
-- LAST_RECON_TIME - The amount of time the last reconcilation took.
-- MAX_RECON_DURATION - maximum duration allowed before timeout of the current 
--                      reconcilation (minutes).
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
-- REQUEST_ID - Tracks the process locking the service
-- CURRENT_REQUEST_COUNT - current number of executing requests
-- MAX_REQUEST_COUNT - maximum number of concurrent executing requests (or -1 = unlimited)
-- LAST_RESPONSE_TIME - timestamp of last response (to detect failed resources)
-- RESOURCE_STATUS - resource status (0 = ok, 1 = failed, 2 = failed service being tested)
--
*/
CREATE TABLE RESOURCE_PROVIDERS 
  (
  PROVIDER_ID                   VARCHAR(20) NOT NULL,
  RESOURCE_DN                   VARCHAR(2000),
  RECON_STATUS                  NUMBER,
  LAST_RECON_TIME               DATE,
  MAX_RECON_DURATION            NUMBER(20),
  LOCK_SERVICE			NUMBER,
  REQUEST_ID                    VARCHAR(20),
  CURRENT_REQUEST_COUNT		NUMBER,
  MAX_REQUEST_COUNT		NUMBER,
  LAST_RESPONSE_TIME		DATE,
  RESOURCE_STATUS		NUMBER,
CONSTRAINT RESOURCE_PROVIDERS_PK PRIMARY KEY(PROVIDER_ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
  )
  TABLESPACE enrole_data;

/**
--
-- REMOTE_SERVICES_REQUESTS
--
-- Table of requests that where made in an asynchronous fasion, or where
-- made while a reconciliation was ongoing.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- REQUEST_ID - the ID of the request that was made.
-- TYPE - The type of request (GENERIC_REQUEST = 0, ASYNCHRONOUS_REQUEST = 1,
--                             INTRA_RECON_REQUEST = 2, SERVICE_DEFERRED_REQUEST = 3)
-- OPERATION - (NOOP = 0, ADD = 1, MODIFY = 2, DELETE = 3, SUSPEND = 4, RESTORE = 5, CHANGE_PASSWORD = 6) 
-- REQUEST_TIME - The time the request was made.
-- EXPIRATION_TIME - The time the request expires. If null the request never expires. (Currently not used.)
-- TARGET - The owner of the account for add or the dn for other operations
-- SERVICE_DN - The distinguished name of the service instance in the directory
-- DATA - The data for the request (attribute values for add and modify).  Serialized Java Collection.
-- CONNECTION_POINT - The callback to complete the workflow process.  Serialized Java object.
--
*/
CREATE TABLE REMOTE_SERVICES_REQUESTS 
  (
  PROVIDER_ID           VARCHAR(20),
  REQUEST_ID            VARCHAR(20) NOT NULL,
  TYPE                  NUMBER,
  OPERATION             NUMBER,
  REQUEST_TIME          DATE,
  EXPIRATION_TIME       DATE,
  TARGET                VARCHAR(2000),
  SERVICE_DN            VARCHAR(2000) NOT NULL,
  DATA			LONG,
  CONNECTION_POINT BLOB,
CONSTRAINT REMOTE_SVR_REQ_FK FOREIGN KEY (PROVIDER_ID)
  REFERENCES RESOURCE_PROVIDERS (PROVIDER_ID),    
CONSTRAINT REMOTE_SVR_REQ_PK PRIMARY KEY(REQUEST_ID) 
    USING INDEX  
    TABLESPACE enrole_indexes    
   )
  TABLESPACE enrole_data;

/**
--
-- REMOTE_RESOURCES_RECONS
--
-- Table of reconciliation units associates with a given resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- DAY_OF_MONTH - Number indicating which day of the month the recon should be
--                run on.
-- MONTH_NUM - Number indicating which month the recon should be run on.
-- DAY_OF_WEEK - Number indicating which day of the week the recon should be run on.
-- DAY_OF_WEEK - Number indicating which hour of the day the recon should be run on.
-- MINUTE_NUM - Number indicating which minute of the hour the recon should be run on.
-- MAX_DURATION - The maximum allowed duration allowed before the recon will time out (minutes)
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
--
*/
CREATE TABLE REMOTE_RESOURCES_RECONS 
  (
  PROVIDER_ID           VARCHAR(20) NOT NULL,
  RECON_ID              NUMBER(20)  NOT NULL,
  DAY_OF_MONTH          NUMBER,
  MONTH_NUM             NUMBER,
  DAY_OF_WEEK           NUMBER,
  HOUR_NUM              NUMBER,
  MINUTE_NUM            NUMBER,
  MAX_DURATION		NUMBER(20),
  LOCK_SERVICE          NUMBER DEFAULT 1,
  CONSTRAINT REMOTE_RES_RECONS_FK FOREIGN KEY(PROVIDER_ID) 
    REFERENCES RESOURCE_PROVIDERS(PROVIDER_ID),
  CONSTRAINT REMOTE_RES_RECONS_PK PRIMARY KEY(PROVIDER_ID, RECON_ID) 
    USING INDEX  
    TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

/**
--
-- REMOTE_RESOURCES_RECON_QUERIES
--
-- Table of reconciliation queries associates with a given reconciliation unit.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider 
--               entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- QUERY_ID - Unique ID for each reconcialation query.
-- RECON_FILTER - Filter associated with this reconciliation query.
-- RECON_BASE - Search base associated with this reconciliation query.
-- MAX_DURATION - not used
-- MAX_ENTRIES - not used
-- ATTRIBUTES - attributes to return in a reconciliation
-- 
*/
CREATE TABLE REMOTE_RESOURCES_RECON_QUERIES 
  (
  PROVIDER_ID           VARCHAR(20) NOT NULL,
  RECON_ID              NUMBER(20)  NOT NULL,
  QUERY_ID              NUMBER(20)  NOT NULL,
  RECON_FILTER          VARCHAR2(4000),
  RECON_BASE            VARCHAR2(4000),
  MAX_DURATION          NUMBER(20),
  MAX_ENTRIES           NUMBER(20),
  ATTRIBUTES            CLOB,
CONSTRAINT REMOTE_RECON_QUERIES_FK FOREIGN KEY(PROVIDER_ID, 
                                               RECON_ID) 
    REFERENCES REMOTE_RESOURCES_RECONS(PROVIDER_ID, 
                                              RECON_ID),
CONSTRAINT REMOTE_RECON_QUERIES_PK PRIMARY KEY(PROVIDER_ID, 
                                               RECON_ID, 
                                               QUERY_ID) 
    USING 
    INDEX  
    TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

/**
--
-- SCHEDULED_MESSAGE 
--
-- Table of scheduled one time or re-occuring events. Usually it contains scheduled user requests submitted
-- by the workflow engine or re-ocurring reconcialation events.
--
-- SCHEDULED_TIME - The long interger that represents the time of the scheduled event, which is
--                  the number of milliseconds since January 1, 1970, 00:00:00 GMT
--
-- SCHEDULED_MESSAGE_ID - Unique ID for each scheduled event
--
-- MESSAGE - A serialized object that represents the detail information of the scheduled event
--
-- SERVER - The server that picks up the scheduled event most recently
--
-- CHECKPOINT_TIME - The long interger that represents the last pick up time of the scheduled event, which
--                   is the number of milliseconds since January 1, 1970, 00:00:00 GMT
--
-- REFERENCE_ID - The is only used for scheduled workflow events, it is the workflow process ID that 
--                sheduled event is coming from.
--
*/
CREATE TABLE SCHEDULED_MESSAGE (
  SCHEDULED_TIME                        NUMBER(20),
  SCHEDULED_MESSAGE_ID                  NUMBER(20) NOT NULL,
  MESSAGE                               Long,
  SERVER                                VARCHAR2(20),
  CHECKPOINT_TIME                       NUMBER(20),
  REFERENCE_ID                          NUMBER(20),
CONSTRAINT SCHEDULED_MESSAGE_PK PRIMARY KEY(
                                       SCHEDULED_MESSAGE_ID) 
    USING 
    INDEX  
    TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE INDEX SCHEDULED_MESSAGE_TIME_X ON SCHEDULED_MESSAGE (SCHEDULED_TIME, SERVER)
  TABLESPACE enrole_indexes;

CREATE INDEX SCHEDULED_MESSAGE_CHECKPOINT_X ON SCHEDULED_MESSAGE (CHECKPOINT_TIME)
  TABLESPACE enrole_indexes;

CREATE INDEX SCHEDULED_MESSAGE_REFERENCE_X ON SCHEDULED_MESSAGE (REFERENCE_ID)
  TABLESPACE enrole_indexes;
/**
-- 
-- NEXTVALUE
-- Table to keep the current maximum value of primary key integer. It only contains
-- one record, for each identfier
--
-- ID -- The next primary key id to be used
--
*/
CREATE TABLE NEXTVALUE 
    (
    ID                  VARCHAR2(50) NOT NULL,
    NEXTID              NUMBER(10) NOT NULL,
CONSTRAINT NEXTVALUE_PK PRIMARY KEY(ID) USING 
     INDEX  
     TABLESPACE enrole_indexes                
    )
  TABLESPACE enrole_data;


COMMIT;
/**
--
-- PROCESS
-- 
-- This table stores all pending, running and historical requests submitted 
-- to the enrole workflow system. Each request is represented as a process.
--
-- ID - The process id
-- PARENT_ID - The parent process id if there is any
-- NAME - The process name
-- TYPE - The process type
-- DEFINITION_ID - The process definition ID
-- REQUESTER_TYPE - The requester type
-- REQUESTER  - The requester identity
-- REQUESTER_NAME - The requester name
-- DESCRIPTION - The process description
-- PRIORITY - The process priority
-- SCHEDULED - The scheduled starting time of the process
-- STARTED - The starting time of the process
-- COMPLETED - The completed time of the process
-- LASTMODIFIED - The last modified time of the process
-- SUBMITTED - The time the process is submitted
-- STATE - The process state
-- COMMENTS - The comments for the process
-- RESULT_SUMMARY - The result summary code of the process
-- RESULT_DETAIL - The result detail information of the process
-- TENANT - The DN of the tenant the requester belongs to
--
*/
CREATE TABLE PROCESS 
      (
                ID                      NUMBER(20) NOT NULL,
                PARENT_ID               NUMBER(20),
                PARENT_ACTIVITY_ID      NUMBER(20),
                NAME                    varchar2(100),
                TYPE                    varchar2(2) NOT NULL,
                DEFINITION_ID           varchar2(2000) NOT NULL,
                REQUESTER_TYPE          varchar2(2),
                REQUESTER               varchar2(2000),
                REQUESTER_NAME          varchar2(100),
                DESCRIPTION             varchar2(300),
                PRIORITY                NUMBER(5),
                SCHEDULED               varchar2(50),
                STARTED                 varchar2(50),
                COMPLETED               varchar2(50),
                LASTMODIFIED            varchar2(50),
                SUBMITTED               VARCHAR2(50),
                STATE                   varchar2(1),
                NOTIFY                  NUMBER(10),
                REQUESTEE               varchar2(2000),
                REQUESTEE_NAME          varchar2(100),
                SUBJECT                 varchar2(2000),
                COMMENTS                varchar2(200),
                RESULT_SUMMARY          varchar2(2),
                RESULT_DETAIL           Long,
                TENANT                  varchar2(2000),
CONSTRAINT PROCESS_PK PRIMARY KEY(ID) USING 
    INDEX  
    TABLESPACE enrole_indexes 
    )
  TABLESPACE enrole_data;

CREATE INDEX PROC_STATE_X ON PROCESS(STATE)
  TABLESPACE enrole_indexes;

CREATE INDEX PROCESS_VPC1_X ON PROCESS(PARENT_ID, SUBMITTED, ID, STATE)
  TABLESPACE enrole_indexes;
  
/**  
--
-- ACTIVITY
--
-- This table contains records to keep track of each process's execution status. 
--
-- ID - The activity id
-- PROCESS_ID - The process id that the activity belongs to
-- DEFINITION_ID - The activity definition ID
-- ACTIVITY_INDEX - The index of the activity if it is inside a loop
-- SUBPROCESS_ID - The ID of the subprocess associated with the activity
-- NAME - The activity name
-- DESCRIPTION - The activity description
-- TYPE - The activity type
-- SUBTYPE - The activity subtype
-- PRIORITY - The activity priority (Currently not used)
-- STARTED - The time the activity started
-- COMPLETED - The time the activity completed
-- LASTMODIFIED - The time the activity was last modified
-- STATE - The activity state
-- RESULT_SUMMARY - The result summary code for the activity
-- RESULT_DETAIL - The result detail for the activity
--
*/
CREATE TABLE ACTIVITY (
                ID                      NUMBER(20) NOT NULL,
                PROCESS_ID              NUMBER(20) NOT NULL,
                DEFINITION_ID           varchar2(100) NOT NULL,
                ACTIVITY_INDEX          NUMBER(10),
                LOOP_COUNT              NUMBER(10),
                LOOP_RUNCOUNT           NUMBER(10),
                RETRY_COUNT             NUMBER(10),
                LOCK_COUNT		NUMBER(10),
                SUBPROCESS_ID           NUMBER(20),
                NAME                    varchar2(100),
                DESCRIPTION             varchar2(300),
                TYPE                    varchar2(1),
                SUBTYPE                 varchar2(2),
                PRIORITY                NUMBER(5),
                STARTED                 varchar2(50),
                COMPLETED               varchar2(50),
                LASTMODIFIED            varchar2(50),
                STATE                   varchar2(1),
                RESULT_SUMMARY          varchar2(2),
                RESULT_DETAIL           Long,
CONSTRAINT ACTIVITY_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT ACTIVITY_PK PRIMARY KEY(ID) USING 
    INDEX  
    TABLESPACE enrole_indexes 
    )
  TABLESPACE enrole_data;

CREATE UNIQUE INDEX ACTIVITY_NATURAL_KEY ON ACTIVITY (PROCESS_ID, DEFINITION_ID, ACTIVITY_INDEX) TABLESPACE enrole_indexes;

/**  
--
-- PROCESSDATA
--
-- This table keeps the runtime process data of a process
-- After process completion, this table will be removed
-- 
-- PROCESS_ID - The process ID associated with the data
-- DEF_ID - The definition ID for the corresponding relevant data in process definition
-- ACTIVITY_ID - The activity ID associated with the data if there is any
-- NAME - The data name
-- DESCRIPTION - The data description
-- TYPE - Data type
-- VALUE - Data value
-- 
*/
CREATE TABLE PROCESSDATA 
(
    PROCESS_ID              NUMBER(20) NOT NULL,
    DEF_ID                  varchar2(100),
    NAME                    varchar2(100),
    CONTEXT                 varchar2(100),
    DESCRIPTION             varchar2(300),
    TYPE                    varchar2(500),
    COLLECTION_TYPE         varchar2(500),
    VALUE                   Long,
    SMALL_VALUE             varchar2(4000),
CONSTRAINT PROCESSDATA_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PROCESSDATA_PK PRIMARY KEY(PROCESS_ID, DEF_ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;
/**   
--
-- PROCESSLOG
--
-- This table keeps records of audit events associated with a process
-- 
-- ID - The processlog ID
-- PROCESS_ID - The ID of the process associated
-- ACTIVITY_ID - The ID of the activity associated
-- CREATED - Time the log record created
-- EVENTTYPE - Log event type
-- OLD_PARTICIPANT_TYPE - Old participant type for assignment change event
-- OLD_PARTICIPANT_ID - Old participant ID for assignment change event
-- NEW_PARTICIPANT_TYPE - New participant type for assignment change event
-- NEW_PARTICIPANT_ID - New participant ID for assignment change event
-- REQUESTOR_TYPE - The requester type for any user related event
-- REQUESTOR - The requester ID for any user related event
-- OLD_STATE - The old state for state change event
-- NEW_STATE - The new state for state change event
-- DATA_ID - The data id for data change event
-- NEW_DATA - The data value for data change event
--
*/
CREATE TABLE PROCESSLOG 
   (
                ID                      NUMBER(20) NOT NULL,
                PROCESS_ID              NUMBER(20),
                ACTIVITY_ID             NUMBER(20),
                CREATED                 varchar2(50),
                EVENTTYPE               varchar2(2) NOT NULL,
                OLD_PARTICIPANT_TYPE    varchar2(2),
                OLD_PARTICIPANT_ID      varchar2(2000),
                NEW_PARTICIPANT_TYPE    varchar2(2),
                NEW_PARTICIPANT_ID      varchar2(2000),
                REQUESTOR_TYPE          varchar2(2),
                REQUESTOR               varchar2(2000),
                OLD_STATE               varchar2(1),
                NEW_STATE               varchar2(1),
                DATA_ID                 varchar2(100),
                NEW_DATA                Long,
CONSTRAINT PROCESSLOG_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PROCESSLOG_PK PRIMARY KEY(ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
    )
  TABLESPACE enrole_data;
/**  
--
-- WORKITEM
-- 
-- This table keeps records of workitems associated with manual workflow activities
-- for running process. The records associated with a process will be removed after 
-- process completion
--
-- ID  -  The workitem ID
-- PROCESS_ID - The process ID associated with the workitem
-- ACTIVITY_ID - The activity ID assciated with the workitem
-- PARTICIPANT_TYPE - The workitem participant type
-- PARTICIPANT  -  The workitem participant identity
-- CREATED - The time the workitem created
--   
*/
CREATE TABLE WORKITEM 
  (
        ID                      NUMBER(20) NOT NULL,
        PROCESS_ID              NUMBER(20) NOT NULL,
        ACTIVITY_ID             NUMBER(20) NOT NULL,
        PARTICIPANT_TYPE        varchar2(2),
        PARTICIPANT             varchar2(2000),
        CREATED                 varchar2(50),
        INPUT_PARAMETERS        Long,
CONSTRAINT WORKITEM2_FK FOREIGN KEY(ACTIVITY_ID) 
    REFERENCES ACTIVITY(ID),
CONSTRAINT WORKITEM_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT WORKITEM_PK PRIMARY KEY(ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE TABLE PASSWORD_TRANSACTION 
  (
        TRANSACTION_ID          NUMBER(20) NOT NULL,
        ACCOUNT_DN                      VARCHAR2(2000),
        CREATION_DATE           VARCHAR2(50),
        PROCESS_ID                      NUMBER(20) NOT NULL,
        ACTIVITY_ID                     NUMBER(20) NOT NULL,
        PASSWORD                        VARCHAR2(500),
CONSTRAINT PASSWORD_TRANSACTION_PK 
PRIMARY KEY(TRANSACTION_ID) USING 
                INDEX  
                TABLESPACE enrole_indexes
  )
TABLESPACE enrole_data;

/**  
--
-- PENDING
--
-- This table keeps the pending provisioning processes
-- 
-- PROCESS_ID - The process ID associated with the data
-- PERSON_DN  - The definition ID for the corresponding relevant data in process definition
-- SERVICE_DN - The activity ID associated with the data if there is any
-- 
*/
CREATE TABLE PENDING 
(
    PROCESS_ID              NUMBER(20) NOT NULL,
    PERSON_DN               varchar2(2000),
    SERVICE_DN              varchar2(2000),
CONSTRAINT PENDING_FK FOREIGN KEY(PROCESS_ID) 
    REFERENCES PROCESS(ID),
CONSTRAINT PENDING_PK PRIMARY KEY(PROCESS_ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

/**
--
-- LISTDATA
--
-- This table is used to persist large data list for performance optimazation
--
-- DATA_ID - The unique identifier of the data
-- INDEX   - The index of the list element
-- VALUE   - The serialized list element
--
*/
CREATE TABLE LISTDATA
(
    DATA_ID  NUMBER(20) NOT NULL,
    INDEX_ID NUMBER(10) NOT NULL,
    VALUE    Long,
CONSTRAINT LISTDATA_PK PRIMARY KEY(DATA_ID, INDEX_ID) USING 
    INDEX  
    TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

CREATE INDEX PENDING_FOR_PERSON ON PENDING (PERSON_DN) TABLESPACE enrole_indexes;

CREATE TABLE REPORT (
	ID INT NOT NULL,
	TITLE VARCHAR(255) NOT NULL, 
	TYPE VARCHAR(255), 
	AUTHOR VARCHAR(255), 
	REPORT_SIZE INT,
	REPORT_DATA LONG RAW,
	STYLESHEET_NAME VARCHAR(255), 
	CONSTRAINT REPORT_P PRIMARY KEY (ID) 
)
TABLESPACE enrole_data;

CREATE TABLE ENTITY_COLUMN (
	ENTITY_NAME VARCHAR(255) NOT NULL, 
	COLUMN_NAME VARCHAR(255) NOT NULL, 
	ATTRIBUTE_NAME VARCHAR(255) NOT NULL, 
	MULTI_VALUED CHAR(1), 
	AVAILABLE_FOR_REPORTING VARCHAR (255), 
	TABLE_NAME VARCHAR (255),
	CONSTRAINT ENTITY_COLUMNP PRIMARY KEY (ENTITY_NAME, ATTRIBUTE_NAME)
)
TABLESPACE enrole_data;


CREATE TABLE COLUMN_REPORT (
	ENTITY_NAME VARCHAR(255) NOT NULL, 
	COLUMN_NAME VARCHAR(255) NOT NULL, 
	REPORT_ID INT NOT NULL, 
	CONSTRAINT COLUMN_REPORT_P PRIMARY KEY (REPORT_ID, ENTITY_NAME, COLUMN_NAME)
)
TABLESPACE enrole_data;


CREATE TABLE ACI (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    SCOPE VARCHAR(255),
    TARGET VARCHAR(255) NOT NULL,
    PARENT VARCHAR(255),
    PRIMARY KEY(DN, NAME, TARGET)
)
TABLESPACE enrole_data;


CREATE TABLE ACI_ROLEDNS (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ROLEDN VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PRINCIPALS (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    PRINCIPALNAME VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PERMISSION_ATTRIBUTERIGHT (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ACTION VARCHAR(6) NOT NULL,
    OPERATION VARCHAR(3) NOT NULL,
    ATTRIBUTERIGHT VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PERMISSION_CLASSRIGHT (
    DN VARCHAR(255) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    TARGET VARCHAR(255) NOT NULL,
    ACTION VARCHAR(6) NOT NULL,
    CLASSRIGHT VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ENTITLEMENT (
    DN VARCHAR(255) NOT NULL,
    TYPE VARCHAR(255) NOT NULL,
    SERVICETARGETTYPE VARCHAR(255),
    SERVICETARGETNAME VARCHAR(255),
    PROCESSDN VARCHAR(255)
)
TABLESPACE enrole_data;

CREATE TABLE ENTITLEMENT_PROVISIONINGPARAMS (
    DN VARCHAR(255) NOT NULL,
    ATTRIBUTEVALUE VARCHAR(4000) NOT NULL,
    NAME VARCHAR(255) NOT NULL,
    ENFORCEMENT VARCHAR(255),
    EXPRTYPE VARCHAR(255)
)
TABLESPACE enrole_data;

CREATE TABLE SYNCHRONIZATION_HISTORY (
    SYNC_ID number(19,0)  NOT NULL ,
    REQUESTOR VARCHAR (255) NOT NULL,
    REQ_TYPE VARCHAR (255),
    REQ_NAME VARCHAR (255),
    STATUS VARCHAR (255),
    TENANT VARCHAR (255),
    STATUS_DETAIL VARCHAR (255),
    SCHEDULED_TIME number(19,0),
    SUBMITTED_TIME number(19,0),
    STARTED_TIME number(19,0) NOT NULL,
    COMPLETED_TIME number(19,0),
    SERVER_NAME VARCHAR (255),
    PRIMARY KEY (SYNC_ID, STARTED_TIME)
)
TABLESPACE enrole_data;

CREATE TABLE SYNCHRONIZATION_LOCK (
    HOST VARCHAR (255) NOT NULL,
    PRIMARY KEY (HOST)
)
TABLESPACE enrole_data;

CREATE TABLE ENROLE.RESOURCES_SYNCHRONIZATIONS (
    SYNC_ID number(19,0)  NOT NULL,
    DAY_OF_MONTH number(10)  NOT NULL,
    MONTH_NUM number(10) NOT NULL,
    DAY_OF_WEEK number(10) NOT NULL,
    HOUR_NUM number(10) NOT NULL,
    MINUTE_NUM number(10) NOT NULL,
    MAX_DURATION number(19,0),
    PRIMARY KEY (SYNC_ID)
)
TABLESPACE enrole_data;

CREATE TABLE CHANGELOG (
	CHANGE_NUMBER number(10) NOT NULL
)
TABLESPACE enrole_data;

/**
--
-- AUTH_KEY
--
-- This table is used to store the keys for signing and verifying authentication requests.
--
-- Y   - The public key in the DSA algorithm
-- P   - The prime number in the DSA algorithm
-- Q   - The sub-prime number in the DSA algorithm
-- G   - The modulus in the DSA algorithm
-- X   - The private key in the DSA algorithm
--
*/
CREATE TABLE AUTH_KEY (
  Y VARCHAR(2000) NOT NULL,
  P VARCHAR(2000) NOT NULL,
  Q VARCHAR(2000) NOT NULL,
  G VARCHAR(2000) NOT NULL,
  X VARCHAR(2000) NOT NULL
) 
TABLESPACE enrole_data;

CREATE TABLE RECONCILIATION (
	RECONID VARCHAR(255) NOT NULL,
	SERVICEDN VARCHAR(2000) NOT NULL,
	PROCESSEDACCOUNTS INTEGER NOT NULL,
	LOCALACCOUNTS INTEGER NOT NULL,
	TIMUSERACCOUNTS INTEGER NOT NULL,
	POLICYVIOLATIONS INTEGER NOT NULL,
	STARTED DATE NOT NULL,
	COMPLETED DATE NOT NULL,
	PRIMARY KEY (RECONID)
)
TABLESPACE enrole_data;

CREATE TABLE RECONCILIATION_INFO (
	RECONID VARCHAR(255) NOT NULL,
	ACCOUNTID VARCHAR(255),
	POLICYCOMPLIANCESTATUS VARCHAR(20),
	USERNAME VARCHAR(255),
	OPERATION VARCHAR(20),
	REMARKS VARCHAR(1000),
	FOREIGN KEY (RECONID) REFERENCES RECONCILIATION (RECONID)
)
TABLESPACE enrole_data;